@@ -8,7 +8,8 @@ check_permission(struct role_acl *role, struct proc_acl *def_acl,
 	struct proc_acl *tmpp = def_acl;
 	struct file_acl *tmpg = NULL;
 	char *tmpname;
-	u_int32_t cap_drop = 0, cap_mask = 0;
+	gr_cap_t cap_drp = {{ 0, 0 }}, cap_mask = {{ 0, 0 }};
+	gr_cap_t cap_full = {{ ~0, ~0 }};
 
 	if (chk->type == CHK_FILE) {
 		tmpname = alloca(strlen(filename) + 1);
@@ -45,17 +46,18 @@ check_permission(struct role_acl *role, struct proc_acl *def_acl,
 		} while (parent_dir(filename, &tmpname));
 	} else if (chk->type == CHK_CAP) {
 		cap_mask = tmpp->cap_mask;
-		cap_drop = tmpp->cap_drop;
+		cap_drp = tmpp->cap_drop;
 
 		while ((tmpp = tmpp->parent_subject)) {
-			cap_drop |= tmpp->cap_drop & (tmpp->cap_mask & ~cap_mask);
-			cap_mask |= tmpp->cap_mask;
+			cap_drp = cap_combine(cap_drp, cap_intersect(tmpp->cap_drop,
+								       cap_drop(tmpp->cap_mask, cap_mask)));
+			cap_mask = cap_combine(cap_mask, tmpp->cap_mask);
 		}
 
-		if (((chk->w_caps == 0xffffffff)
-		     || !(cap_drop & chk->w_caps))
-		    && ((chk->u_caps == 0xffffffff)
-			|| (cap_drop & chk->u_caps)))
+		if (((cap_same(chk->w_caps, cap_full))
+		     || cap_isclear(cap_intersect(cap_drp, chk->w_caps)))
+		    && ((cap_same(chk->u_caps, cap_full))
+			|| !cap_isclear(cap_intersect(cap_drp, chk->u_caps))))
 			return 1;
 	}
 
@@ -153,6 +155,41 @@ check_default_objects(struct role_acl *role)
 	return;
 }
 
+static void
+check_socket_policies(struct role_acl *role)
+{
+	struct proc_acl *tmp;
+	struct ip_acl *tmpi;
+	int has_connect;
+	int has_bind;
+	unsigned int i;
+
+	for_each_subject(tmp, role) {
+		has_connect = 0;
+		has_bind = 0;
+		for (i = 0; i < tmp->ip_num; i++) {
+			tmpi = tmp->ips[i];
+			if (tmpi->mode & GR_IP_BIND)
+				has_bind = 1;
+			if (tmpi->mode & GR_IP_CONNECT)
+				has_connect = 1;
+		}
+		/* if we have either a bind or a connect, but not either
+		   and not both */
+		if (has_bind ^ has_connect) {
+			fprintf(stderr, "A %s rule exists but a %s rule is missing for "
+				"role %s subject %s\nThe RBAC system will "
+				"not load until you correct this "
+				"error.\n", has_connect ? "connect" : "bind",
+				has_bind ? "connect" : "bind",  
+				role->rolename, tmp->filename);
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	return;
+}
+
 static int
 check_lilo_conf(struct role_acl *role, struct proc_acl *def_acl)
 {
@@ -248,7 +285,8 @@ check_path_env(struct role_acl *role, struct proc_acl *def_acl)
 	while ((p2 = strchr(p, ':'))) {
 		*p2++ = '\0';
 		memset(pathbuf, 0, sizeof(pathbuf));
-		realpath(p, pathbuf);
+		if (!realpath(p, pathbuf))
+			goto next;
 		if (!stat(pathbuf, &fstat)
 		    && !check_permission(role, def_acl, pathbuf, &chk)) {
 			fprintf(stderr,
@@ -258,11 +296,14 @@ check_path_env(struct role_acl *role, struct proc_acl *def_acl)
 				role->rolename, pathbuf);
 			errs_found++;
 		}
+next:
 		p = p2;
 	}
 
 	memset(pathbuf, 0, sizeof(pathbuf));
-	realpath(p, pathbuf);
+	if (!realpath(p, pathbuf))
+		goto reterr;
+
 	if (!stat(pathbuf, &fstat) && !check_permission(role, def_acl, pathbuf, &chk)) {
 		fprintf(stderr,
 			"Write access is allowed by role %s to %s, a directory which "
@@ -272,6 +313,7 @@ check_path_env(struct role_acl *role, struct proc_acl *def_acl)
 		errs_found++;
 	}
 
+reterr:
 	return errs_found;
 }
 
@@ -363,6 +405,13 @@ check_role_transitions(void)
 	}
 
 	for_each_role(role, current_role) {
+		if (role->transitions && !(role->roletype & (GR_ROLE_SPECIAL | GR_ROLE_AUTH))) {
+			fprintf(stderr, "Error in role %s: a transition to a special role exists, "
+					"but the \"G\" flag is not present on the role to grant it "
+					"permission to use gradm to change to the special role.\n",
+					role->rolename);
+			errors++;
+		}
 		for_each_transition(trans, role->transitions) {
 			found = 0;
 			for_each_role(role2, current_role) {
@@ -412,6 +461,7 @@ analyze_acls(void)
 	struct role_acl *role;
 	int def_role_found = 0;
 	struct stat fstat;
+	gr_cap_t cap_full = {{ ~0, ~0 }};
 
 	errs_found = check_role_transitions();
 
@@ -440,6 +490,7 @@ analyze_acls(void)
 		}
 
 		check_default_objects(role);
+		check_socket_policies(role);
 
 		/* non-critical warnings aren't issued for special roles */
 		if (role->roletype & GR_ROLE_SPECIAL)
@@ -510,6 +561,14 @@ analyze_acls(void)
 			errs_found++;
 		}
 
+		if (!stat("/lib/modules", &fstat) && !check_permission(role, def_acl, "/lib/modules", &chk)) {
+			fprintf(stderr,
+				"Writing access is allowed by role %s to /lib/modules, the directory which "
+				"holds kernel modules.\n\n",
+				role->rolename);
+			errs_found++;
+		}
+
 		if (!check_permission(role, def_acl, "/dev", &chk)) {
 			fprintf(stderr,
 				"Writing access is allowed by role %s to /dev, the directory which "
@@ -605,9 +664,66 @@ analyze_acls(void)
 			errs_found++;
 		}
 
+		if (!stat("/proc/slabinfo", &fstat) && !check_permission(role, def_acl, "/proc/slabinfo", &chk)) {
+			fprintf(stderr,
+				"Reading access is allowed by role %s to "
+				"/proc/slabinfo, an entry that provides "
+				"useful information to an attacker "
+				"for reliable heap exploitation in the "
+				"kernel.\n\n", role->rolename);
+			errs_found++;
+		}
+
+		if (!stat("/proc/modules", &fstat) && !check_permission(role, def_acl, "/proc/modules", &chk)) {
+			fprintf(stderr,
+				"Reading access is allowed by role %s to "
+				"/proc/modules, an entry that provides "
+				"useful kernel addresses to an attacker "
+				"for reliable exploitation of the "
+				"kernel.\n\n", role->rolename);
+			errs_found++;
+		}
+
+		if (!check_permission(role, def_acl, "/boot", &chk)) {
+			fprintf(stderr,
+				"Reading access is allowed by role %s to "
+				"/boot, the directory which holds kernel "
+				"images.  The ability to read these "
+				"images provides an attacker with very "
+				"useful information for launching \"ret-to-libc\" "
+				"style attacks against the kernel"
+				".\n\n", role->rolename);
+			errs_found++;
+		}
+
+		if (!stat("/lib/modules", &fstat) && !check_permission(role, def_acl, "/lib/modules", &chk)) {
+			fprintf(stderr,
+				"Reading access is allowed by role %s to "
+				"/lib/modules, the directory which holds kernel "
+				"kernel modules.  The ability to read these "
+				"images provides an attacker with very "
+				"useful information for launching \"ret-to-libc\" "
+				"style attacks against the kernel"
+				".\n\n", role->rolename);
+			errs_found++;
+		}
+
+		if (!stat("/proc/kallsyms", &fstat) && !check_permission(role, def_acl, "/proc/kallsyms", &chk)) {
+			fprintf(stderr,
+				"Reading access is allowed by role %s to "
+				"/proc/kallsyms, a pseudo-file that "
+				"holds a mapping between kernel "
+				"addresses and symbols.  This information "
+				"is very useful to an attacker in "
+				"sophisticated kernel exploits.\n\n",
+				role->rolename);
+			errs_found++;
+		}
+
 		chk.type = CHK_CAP;
-		chk.u_caps = cap_conv("CAP_SYS_MODULE") | cap_conv("CAP_SYS_RAWIO") | cap_conv("CAP_MKNOD");
-		chk.w_caps = 0xffffffff;
+		chk.u_caps = cap_combine(cap_combine(cap_conv("CAP_SYS_MODULE"), cap_conv("CAP_SYS_RAWIO")), 
+					 cap_conv("CAP_MKNOD"));
+		chk.w_caps = cap_full;
 
 		if (!check_permission(role, def_acl, "", &chk)) {
 			fprintf(stderr,
@@ -620,7 +736,7 @@ analyze_acls(void)
 		}
 
 		chk.u_caps = cap_conv("CAP_SYS_ADMIN");
-		chk.w_caps = 0xffffffff;
+		chk.w_caps = cap_full;
 
 		if (!check_permission(role, def_acl, "", &chk)) {
 			fprintf(stderr, "CAP_SYS_ADMIN is not "
@@ -632,18 +748,18 @@ analyze_acls(void)
 
 
 		chk.u_caps = cap_conv("CAP_SYS_BOOT");
-		chk.w_caps = 0xffffffff;
+		chk.w_caps = cap_full;
 
 		if (!check_permission(role, def_acl, "", &chk)) {
 			fprintf(stderr, "CAP_SYS_BOOT is not "
 				"removed in role %s.  This would allow an "
-				"attacker to reboot the system.\n\n",
+				"attacker to reboot the system or to load a new kernel through the kexec interface .\n\n",
 				role->rolename);
 			errs_found++;
 		}
 
 		chk.u_caps = cap_conv("CAP_NET_ADMIN");
-		chk.w_caps = 0xffffffff;
+		chk.w_caps = cap_full;
 
 		if (!check_permission(role, def_acl, "", &chk)) {
 			fprintf(stderr, "CAP_NET_ADMIN is not "
@@ -654,7 +770,7 @@ analyze_acls(void)
 		}
 
 		chk.u_caps = cap_conv("CAP_NET_BIND_SERVICE");
-		chk.w_caps = 0xffffffff;
+		chk.w_caps = cap_full;
 
 		if (!check_permission(role, def_acl, "", &chk)) {
 			fprintf(stderr, "CAP_NET_BIND_SERVICE is not "
@@ -666,7 +782,7 @@ analyze_acls(void)
 		}
 
 		chk.u_caps = cap_conv("CAP_SYS_TTY_CONFIG");
-		chk.w_caps = 0xffffffff;
+		chk.w_caps = cap_full;
 
 		if (!check_permission(role, def_acl, "", &chk)) {
 			fprintf(stderr, "CAP_SYS_TTY_CONFIG is not "
