@@ -12,7 +12,13 @@ void analyze_acls(void);
 void generate_hash(struct gr_pw_entry *entry);
 void init_variables(void);
 void parse_args(int argc, char *argv[]);
-u_int32_t cap_conv(const char *cap);
+gr_cap_t cap_conv(const char *cap);
+gr_cap_t cap_drop(gr_cap_t a, gr_cap_t b);
+gr_cap_t cap_combine(gr_cap_t a, gr_cap_t b);
+gr_cap_t cap_intersect(gr_cap_t a, gr_cap_t b);
+int cap_same(gr_cap_t a, gr_cap_t b);
+gr_cap_t cap_invert(gr_cap_t a);
+int cap_isclear(gr_cap_t a);
 u_int32_t file_mode_conv(const char *mode);
 u_int32_t proc_subject_mode_conv(const char *mode);
 u_int32_t proc_object_mode_conv(const char *mode);
@@ -152,4 +158,8 @@ void parse_learn_config(void);
 
 void check_pam_auth(unsigned char *rolename);
 
+void add_replace_string(char *name, char *replacewith);
+char *lookup_replace_string(char *name);
+char *process_string_replace(char *str);
+
 #endif
