@@ -138,8 +138,12 @@ void merge_acl_rules(void)
 
 				matchsubj->subject->pax_flags = subject->pax_flags;
 
-				matchsubj->subject->cap_raise |= ~subject->cap_drop;
+				matchsubj->subject->cap_raise = cap_combine(matchsubj->subject->cap_raise,
+									    cap_invert(subject->cap_drop));
 				matchsubj->subject->resmask |= subject->resmask;
+
+				matchsubj->subject->inaddr_any_override = subject->inaddr_any_override;
+
 				for (i = 0; i < subject->user_trans_num; i++) {
 					x = *(subject->user_transitions + i);
 					insert_learn_id_transition(&(matchsubj->user_trans_list), x, x, x);
