@@ -498,7 +498,8 @@ int reduce_all_children(struct gr_learn_file_node *node)
 		if (!((*tmp)->leaves)) {
 			node->mode |= (*tmp)->mode;
 			if (node->subject && (*tmp)->subject) {
-				node->subject->cap_raise |= (*tmp)->subject->cap_raise;
+				node->subject->cap_raise = cap_combine(node->subject->cap_raise, 
+								       (*tmp)->subject->cap_raise);
 				node->subject->resmask |= (*tmp)->subject->resmask;
 				for (i = 0; i < GR_NLIMITS; i++) {
 					if ((*tmp)->subject->res[i].rlim_cur > node->subject->res[i].rlim_cur)
@@ -550,7 +551,8 @@ int reduce_all_leaves(struct gr_learn_file_node *node)
 		reduce_all_leaves(*tmp);
 		node->mode |= (*tmp)->mode;
 		if (node->subject && (*tmp)->subject) {
-			node->subject->cap_raise |= (*tmp)->subject->cap_raise;
+			node->subject->cap_raise = cap_combine(node->subject->cap_raise,
+							       (*tmp)->subject->cap_raise);
 			node->subject->resmask |= (*tmp)->subject->resmask;
 			for (i = 0; i < GR_NLIMITS; i++) {
 				if ((*tmp)->subject->res[i].rlim_cur > node->subject->res[i].rlim_cur)
@@ -1165,18 +1167,18 @@ int display_leaf(struct gr_learn_file_node *node,
 		}
 
 		for(i = raise_num = 0; i < ((sizeof(capability_list)/sizeof(struct capability_set)) - 1); i++)
-			if (node->subject->cap_raise & (1 << capability_list[i].cap_val))
+			if (cap_raised(node->subject->cap_raise, capability_list[i].cap_val))
 				raise_num++;
 
 		if (raise_num < ((sizeof(capability_list)/sizeof(struct capability_set)) - 1) / 2) {
 			fprintf(stream, "\t-CAP_ALL\n");
 			for(i = 0; i < ((sizeof(capability_list)/sizeof(struct capability_set)) - 1); i++)
-				if (node->subject->cap_raise & (1 << capability_list[i].cap_val))
+				if (cap_raised(node->subject->cap_raise, capability_list[i].cap_val))
 					fprintf(stream, "\t+%s\n", capability_list[i].cap_name);
 		} else {
 			fprintf(stream, "\t+CAP_ALL\n");
 			for(i = 0; i < ((sizeof(capability_list)/sizeof(struct capability_set)) - 1); i++)
-				if (!(node->subject->cap_raise & (1 << capability_list[i].cap_val)))
+				if (!cap_raised(node->subject->cap_raise, capability_list[i].cap_val))
 					fprintf(stream, "\t-%s\n", capability_list[i].cap_name);
 		}
 
@@ -1202,6 +1204,11 @@ show_ips:
 			display_ip_tree(connect, GR_IP_CONNECT, stream);
 		else
 			fprintf(stream, "\tconnect\tdisabled\n");
+		if (node->subject != NULL && node->subject->inaddr_any_override) {
+			struct in_addr addr;
+			addr.s_addr = node->subject->inaddr_any_override;
+			fprintf(stream, "\tip_override\t%s\n", inet_ntoa(addr));
+		}
 
 		fprintf(stream, "}\n\n");
 	} else {
@@ -1782,7 +1789,7 @@ insert_learn_role(struct gr_learn_role_entry ***role_list, char *rolename, u_int
 	(*((*role_list) + num))->rolemode = rolemode;
 
 	/* give every learned role a / subject */
-	insert_learn_role_subject(*((*role_list) + num), conv_filename_to_struct("/", GR_FIND));
+	insert_learn_role_subject(*((*role_list) + num), conv_filename_to_struct("/", GR_PROCFIND | GR_OVERRIDE));
 
 	return (*((*role_list) + num));
 }
