@@ -26,14 +26,21 @@ add_id_transition(struct proc_acl *subject, char *idname, int usergroup, int all
 				return;
 
 		pwd = getpwnam(idname);
-
 		if (!pwd) {
-			fprintf(stderr, "User %s on line %lu of %s "
-				"does not exist.\nThe RBAC system will "
-				"not be allowed to be enabled until "
-				"this error is fixed.\n", idname,
-				lineno, current_acl_file);
-			exit(EXIT_FAILURE);
+			/* now try it as a uid */
+			unsigned long theuid = 0;
+			char *endptr;
+			theuid = strtoul(idname, &endptr, 10);
+			if (*endptr == '\0')
+				pwd = getpwuid((int)theuid);
+			if (!pwd || theuid > INT_MAX) {
+				fprintf(stderr, "User %s on line %lu of %s "
+					"does not exist.\nThe RBAC system will "
+					"not be allowed to be enabled until "
+					"this error is fixed.\n", idname,
+					lineno, current_acl_file);
+				exit(EXIT_FAILURE);
+			}
 		}
 
 		/* increment pointer count upon allocation of user transition list */
@@ -61,12 +68,20 @@ add_id_transition(struct proc_acl *subject, char *idname, int usergroup, int all
 		grp = getgrnam(idname);
 
 		if (!grp) {
-			fprintf(stderr, "Group %s on line %lu of %s "
-				"does not exist.\nThe RBAC system will "
-				"not be allowed to be enabled until "
-				"this error is fixed.\n", idname,
-				lineno, current_acl_file);
-			exit(EXIT_FAILURE);
+			/* now try it as a gid */
+			unsigned long thegid = 0;
+			char *endptr;
+			thegid = strtoul(idname, &endptr, 10);
+			if (*endptr == '\0')
+				grp = getgrgid((int)thegid);
+			if (!grp || thegid > INT_MAX) {
+				fprintf(stderr, "Group %s on line %lu of %s "
+					"does not exist.\nThe RBAC system will "
+					"not be allowed to be enabled until "
+					"this error is fixed.\n", idname,
+					lineno, current_acl_file);
+				exit(EXIT_FAILURE);
+			}
 		}
 
 		/* increment pointer count upon allocation of group transition list */
@@ -122,12 +137,20 @@ add_domain_child(struct role_acl *role, char *idname)
 		pwd = getpwnam(idname);
 
 		if (!pwd) {
-			fprintf(stderr, "User %s on line %lu of %s "
-				"does not exist.\nThe RBAC system will "
-				"not be allowed to be enabled until "
-				"this error is fixed.\n", idname,
-				lineno, current_acl_file);
-			exit(EXIT_FAILURE);
+			/* now try it as a uid */
+			unsigned long theuid = 0;
+			char *endptr;
+			theuid = strtoul(idname, &endptr, 10);
+			if (*endptr == '\0')
+				pwd = getpwuid((int)theuid);
+			if (!pwd || theuid > INT_MAX) {
+				fprintf(stderr, "User %s on line %lu of %s "
+					"does not exist.\nThe RBAC system will "
+					"not be allowed to be enabled until "
+					"this error is fixed.\n", idname,
+					lineno, current_acl_file);
+				exit(EXIT_FAILURE);
+			}
 		}
 
 		role->domain_child_num++;
@@ -137,12 +160,20 @@ add_domain_child(struct role_acl *role, char *idname)
 		grp = getgrnam(idname);
 
 		if (!grp) {
-			fprintf(stderr, "Group %s on line %lu of %s "
-				"does not exist.\nThe RBAC system will "
-				"not be allowed to be enabled until "
-				"this error is fixed.\n", idname,
-				lineno, current_acl_file);
-			exit(EXIT_FAILURE);
+			/* now try it as a gid */
+			unsigned long thegid = 0;
+			char *endptr;
+			thegid = strtoul(idname, &endptr, 10);
+			if (*endptr == '\0')
+				grp = getgrgid((int)thegid);
+			if (!grp || thegid > INT_MAX) {
+				fprintf(stderr, "Group %s on line %lu of %s "
+					"does not exist.\nThe RBAC system will "
+					"not be allowed to be enabled until "
+					"this error is fixed.\n", idname,
+					lineno, current_acl_file);
+				exit(EXIT_FAILURE);
+			}
 		}
 
 		role->domain_child_num++;
@@ -206,7 +237,7 @@ add_deleted_file(char *filename)
 {
 	struct deleted_file *dfile;
 	struct deleted_file *retfile;
-	static ino_t ino = 0;
+	static ino_t ino = 0x10000000;
 
 	ino++;
 
@@ -310,12 +341,20 @@ add_role_acl(struct role_acl **role, char *rolename, u_int16_t type, int ignore)
 			pwd = getpwnam(rolename);
 
 			if (!pwd) {
-				fprintf(stderr, "User %s on line %lu of %s "
-					"does not exist.\nThe RBAC system will "
-					"not be allowed to be enabled until "
-					"this error is fixed.\n", rolename,
-					lineno, current_acl_file);
-				exit(EXIT_FAILURE);
+				/* now try it as a uid */
+				unsigned long theuid = 0;
+				char *endptr;
+				theuid = strtoul(rolename, &endptr, 10);
+				if (*endptr == '\0')
+					pwd = getpwuid((int)theuid);
+				if (!pwd || theuid > INT_MAX) {
+					fprintf(stderr, "User %s on line %lu of %s "
+						"does not exist.\nThe RBAC system will "
+						"not be allowed to be enabled until "
+						"this error is fixed.\n", rolename,
+						lineno, current_acl_file);
+					exit(EXIT_FAILURE);
+				}
 			}
 
 			rtmp->uidgid = pwd->pw_uid;
@@ -323,12 +362,20 @@ add_role_acl(struct role_acl **role, char *rolename, u_int16_t type, int ignore)
 			grp = getgrnam(rolename);
 
 			if (!grp) {
-				fprintf(stderr, "Group %s on line %lu of %s "
-					"does not exist.\nThe RBAC system will "
-					"not be allowed to be enabled until "
-					"this error is fixed.\n", rolename,
-					lineno, current_acl_file);
-				exit(EXIT_FAILURE);
+				/* now try it as a gid */
+				unsigned long thegid = 0;
+				char *endptr;
+				thegid = strtoul(rolename, &endptr, 10);
+				if (*endptr == '\0')
+					grp = getgrgid((int)thegid);
+				if (!grp || thegid > INT_MAX) {
+					fprintf(stderr, "Group %s on line %lu of %s "
+						"does not exist.\nThe RBAC system will "
+						"not be allowed to be enabled until "
+						"this error is fixed.\n", rolename,
+						lineno, current_acl_file);
+					exit(EXIT_FAILURE);
+				}
 			}
 
 			rtmp->uidgid = grp->gr_gid;
@@ -527,6 +574,7 @@ add_proc_object_acl(struct proc_acl *subject, char *filename,
 	struct deleted_file *dfile;
 	unsigned int file_len;
 	char *str;
+	static int link_count = 0;
 
 	if (!subject) {
 		fprintf(stderr, "Error on line %lu of %s.  Attempt to "
@@ -584,12 +632,23 @@ add_proc_object_acl(struct proc_acl *subject, char *filename,
 		fstat.st_ino = dfile->ino;
 		fstat.st_dev = 0;
 		mode |= GR_DELETED;
+		link_count = 0;
 	} else if (S_ISLNK(fstat.st_mode)) {
-		char buf[PATH_MAX];
-		memset(&buf, 0, sizeof (buf));
-		realpath(filename, buf);
-		if(!add_proc_object_acl(subject, gr_strdup(buf), mode, type | GR_SYMLINK))
-			return 0;
+		if (link_count > MAX_SYMLINK_DEPTH) {
+			fprintf(stderr, "Error: Too many levels of symbolic links when accessing "
+					"%s\n", filename);
+			exit(EXIT_FAILURE);
+		} else {
+			char buf[PATH_MAX];
+			memset(&buf, 0, sizeof (buf));
+			if (!realpath(filename, buf)) {
+				fprintf(stderr, "Error determining real path for %s\n", filename);
+				exit(EXIT_FAILURE);
+			}
+			link_count++;
+			if(!add_proc_object_acl(subject, gr_strdup(buf), mode, type | GR_SYMLINK))
+				return 0;
+		}
 	}
 
 	if ((p =
@@ -636,7 +695,7 @@ add_proc_object_acl(struct proc_acl *subject, char *filename,
 			subject->filename, lineno, 
 			current_acl_file ? current_acl_file : "<builtin_fulllearn_policy>", p->filename);
 		display_all_dupes(subject, p);
-		fprintf(stderr, "specified on an earlier line."
+		fprintf(stderr, "specified on an earlier line.  "
 			"The RBAC system will not load until this"
 			" error is fixed.\n");
 		exit(EXIT_FAILURE);
